var mensaje = require('js2xmlparser');
var xml2js = require('xml2js').parseString;
var net = require('net');
var global = {	conn: new net.Socket(),
				last_id: '',
				last_cmd: '',
				curr_state: 'wait',
				buffer: ''
			 };
var defaults = {
	'receive_lock' : function(data, callback) {
		global.last_id = data['$'].id;
		console.log('*** recibimos lock id: ****',global.last_id);
		if (callback) callback(data);
	}
};

exports.getLastID = function() {
	return global.last_id;
};

exports.parseXml = function(xml, callback) {
	xml2js(xml, function(err, result) {
		var parent_tag = Object.keys(result)[0].toString();
		callback(parent_tag, result[parent_tag]);
	});
};

exports.init = function(callback, options) {
	if (options) {
		// definimos valores default, en caso de haber options definidos
		if ('retry' in options) {
			if (options.retry==true) {
				defaults['goodbye'] = function() {
					console.log('conn:retrying connecting as requested in options.');
					exports.init();
				}
			}
		}
		//
	}
	global.conn.connect(9000, 'localhost', function() {
		callback();
	});
};

exports.on = function(events) {
	global.conn.on('data', function(data) {
		var xmlheader = 'encoding="UTF-8"?>';
		var msg_continues = false;
		if (data.toString().indexOf('<cont>')!=-1) {
			// este pack de datos necesita sumarse al buffer
			var tt = data.toString().split('<cont>').join('');
			global.buffer += tt;
			msg_continues = true;
		} else {
			global.buffer = data.toString();
		}
		var justcmd = global.buffer;
		var pos = justcmd.toString().indexOf('encoding="UTF-8"?>')+xmlheader.length;
		if (pos!=-1) {
			justcmd = justcmd.slice(pos);
		}
		if (msg_continues==false) {
			try {
				var recibido = xml2js(justcmd, function(err, result) {
					var cmd = Object.keys(result)[0].toString();
					var scmd = cmd.split('collaboration_').join('');
					if (err) console.log('error en xml2js:',err);
					global.last_cmd = scmd;
					if (scmd in events) {
						//console.log('conn:llamando cmd:'+scmd+' definido en param events');
						events[scmd](result[cmd]);
					} else if ('*' in events) {
						// evento fallback en si mismo detectado
						if (scmd in defaults) {
							defaults[scmd](result[cmd],function(x) {
								events['*'](result[cmd],cmd);
							});
						} else {
							events['*'](result[cmd],cmd);
						}
					} else if (scmd in defaults) {
						defaults[scmd](result[cmd]);
						//console.log('conn:cmd:'+scmd+' no ha sido definido en param events. se omite.');
					}
				});
			} catch(e) {
				console.log('error procesando respuesta xml a JS. xml:', justcmd);
				console.log(e);
			}
		} 
	});
};

exports.write = function(cmd, obj) {
	var tmp = { cmd: 'collaboration_'+cmd, data:obj };
	var msg = mensaje(tmp.cmd, obj);
	//msg = msg.split('&gt;').join('>');
	msg = msg.split('\n').join('');
	console.log('\nconn:send:enviando:',msg);
	global.conn.write(msg+'\r\n','utf-8', function() {
		//global.conn.end();
	});
};

exports.send = {
	login : function(name) {
		exports.write('hello',	{ "@" : {	"user_id"	: name+"@MacBookdeX.home", 
											"password"	: "depurador123", 
										} 
								} 
					 );
	},
	getlock : function() {
		exports.write('require_lock',{});
	},
	transaction : function(action,undo) {
		var espera = 50;
		if (global.last_cmd!='receive_lock' || global.last_id=='') {
			exports.send.getlock();
		    espera = 1000;
		}
		setTimeout(function() {
			// ejecutamos con desfase segun si hay o no lock.
			exports.write('transaction',{
					'@' : {
						'id' 		 	 :  global.last_id,
						'do_action'		 :  mensaje('compound_action',action),
						'undo_action'	 :  mensaje('compound_action',undo)
					}
			});
		}, espera);
	}
};

exports.destroy = function() {
	global.conn.destroy();
};