var concepto = require('../../concepto');
var docs = {
	info	:  {
		title		:  'Titanium Concepto DSL Manual',
		covertext	:  'DSL Module for creating native mobile applications using Appcelerator Titanium Framework',
		language	:  'english',
		maqueta		:  {},
		revisions	:  [
			{
				'date'   		: '29-12-2015',
				'author'	 	: 'Pablo Schaffner',
				'type'		 	: 'new',
				'description'	: 'First NodeJS version'
			}
		],
		interchapters	:  [
			{
				title		: 'Introduction',
				content		: 'Titanium DSL module for Concepto DSL represents a revolution in native mobile app generation\
							   by means of a structured and easy to use way.'
			}
		],
		afterchapters	:  [
		],
		central			:  {
			hint		:  'Central Node',
			text		:  'The central node text content defines the application title and its attributes define the global configuration of the compiler.',
			attributes	:  [
				{
					required 	: true,
					name		: 'devices',
					values		: 'auto, iphone, ipad, android device name, mobileweb',
					description	: 'Defines one or multiple target devices for which to generate code'
				},
				{
					required	: false,
					name		: 'sdk',
					values		: 'last, 3.1.3.GA or specific Titanium SDK version to be used',
					description	: 'The Titanium SDK version to use.'
				}
			]
		}
	}
};

var config = {
	'text_breakline'		:  '<br/>',
	'cmd_breakline'			:  '\n',
	'textvar_output_open'	:  '<varopen>',
	'textvar_output_close'	:  '</varopen>',
	'text_output_open'		:  '',
	'text_output_close'		:  '',
	'text_output_to_omit'	:  {
		'face'	:  'SansSerif',
		'size'	:  '12'
	},
	'text_format'			:  {
		'bold'		:  {
			'if'	:  {	'tag':'b',	'node':'font.bold'	},
			'type'	:  'wrap',
			'tag'	:  'b'
		},
		'italic'	:  {
			'if'	:  {	'tag':'i',	'node':'font.italic'	},
			'type'	:  'wrap',
			'open'	:  '<i>',
			'close'	:  '</i>'
		}
	}
};

var methods = {
	'info'		:  function() {
		var infox = docs.info;
		infox.x_config = config;
		return infox;
	},
	'onInit'	:  function(o) {
	},
};

var styles = {

};

var nodes = [
	{
		match		:  {
			icons		:  'button_cancel'	
		},
		info		:  {
			'id'		:  'cancel',
			'hint'		:  'excludes any node and its children of being processed.',
			'doc'		:  {
				'description'	:  'Excludes any node and its children of being processed.'
			}
		},
		'return'	:  function(node) {
			return concepto.reply_template({ hasChildren: false });
		}
	},
	{
		info 		:  {
			'id'		:  'def_window',
			'hint'		:  'Screen node'
		},
		match 		:  {
			not_icons		:  'forward,desktop_new',
			level			:  2
		},
		'attributes'	:  {
			'nav,links,navegacion,enlaces,tipo'		:  {
				'required'	:  false,
				'default'	:  'normal',
				'values'	:  {
					'internos,interior,interiores,incluir,incluidos,internal,interna,agrupada,agrupados,navgroup' : 'navgroup'
				},
				'hint'		:  'IOS exlusive, defines that inner links have to replace its parent content',
				'mapto'		:  '_type',
			},
			'titulo,title'		:  {
				'required'	:  false,
				'default'	:  'node.text',
				'values'	:  '*',
				'hint'		:  'The window title, if undefined, uses the node text as the title.',
				'mapto'		:  'title',
				'smart'		:  {
					// smart plugin support, executed by 'smart.js' connection plugin
					'onChange'	:  function(node, this_attr) {
						return node.id+'.setTitle('+this_attr.value+');';
					}
				}
			},
			'layout'			:   {
				'required'	:  false,
				'default'	:  'vertical',
				'values'	:  {
					'vertical'						:  'vertical',
					'horizontal'					:  'horizontal',
					'composite,absolute,absoluto'	:  'composite'
				},
				'hint'		:  'Defines the way to stack its children.',
				'mapto'		:  'layout'
			},
			'fondo,background'	:  {
				'required'	:  false,
				'default' 	:  '',
				'values'	:  {
					'*'		:   function(value) {
						if (value in styles['images']) {
							return styles['images'][value];
						} else {
							return '';
						}
					}
				},
				'hint'		:  'ID of image style to be applied as a background image for this window.',
				'mapto'		:  'backgroundImage'
			}
		},
		'return'	:  function(node, attr) {
			// attr: already processed attributes[] (ex: { name:'_type', value:'navgroup'} ).
			var tmp = {};
			var resp = concepto.reply_template({ hasChildren: true });
			var params = {  
				'id'		:  node.id
			};
			if (node.bgcolor!='') params['backgroundColor']=node.bgcolor;
			if (node.color!='') params['navTintColor']=node.color;
			for (var p in attr) {
				if (p.charAt(0)!='_') {
					params[p] = attr[p];
				} else {
					tmp[p] = attr[p];
				}
			}
			// renders the node content
			//
			return resp;
			//
		}
	}
];
